searchData={"content_type":"text/markdown","items":[{"doc":"Params provide some helpers method to work with parameters","ref":"Skema.html","title":"Skema","type":"module"},{"doc":"","ref":"Skema.html#cast/2","title":"Skema.cast/2","type":"function"},{"doc":"Cast and validate params with given schema.\nSee `Skema.SchemaHelper` for instruction on how to define a schema\nAnd then use it like this\n\n```elixir\ndef index(conn, params) do\n  index_schema = %{\n    status: [type: :string, required: true],\n    type: [type: :string, in: [\"type1\", \"type2\", \"type3\"]],\n    keyword: [type: :string, length: [min: 3, max: 100]],\n  }\n\n  with {:ok, data} <- Skema.cast(params, index_schema) do\n    # do query data\n  else\n    {:error, errors} -> IO.puts(errors)\n  end\nend\n```","ref":"Skema.html#cast_and_validate/2","title":"Skema.cast_and_validate/2","type":"function"},{"doc":"","ref":"Skema.html#cast_and_validate!/2","title":"Skema.cast_and_validate!/2","type":"function"},{"doc":"Cast and validate params with given schema.","ref":"Skema.html#cast_apply/2","title":"Skema.cast_apply/2","type":"function"},{"doc":"","ref":"Skema.html#cast_array/3","title":"Skema.cast_array/3","type":"function"},{"doc":"","ref":"Skema.html#transform/1","title":"Skema.transform/1","type":"function"},{"doc":"","ref":"Skema.html#transform/2","title":"Skema.transform/2","type":"function"},{"doc":"Transform params with given schema.","ref":"Skema.html#transform_apply/2","title":"Skema.transform_apply/2","type":"function"},{"doc":"","ref":"Skema.html#validate/1","title":"Skema.validate/1","type":"function"},{"doc":"","ref":"Skema.html#validate/2","title":"Skema.validate/2","type":"function"},{"doc":"Validate params with given schema.","ref":"Skema.html#validate_apply/2","title":"Skema.validate_apply/2","type":"function"},{"doc":"Result Struct for Skema operations","ref":"Skema.Result.html","title":"Skema.Result","type":"module"},{"doc":"Get error from result for given field.","ref":"Skema.Result.html#get_error/2","title":"Skema.Result.get_error/2","type":"function"},{"doc":"Create a new Result struct with given schema map and params.","ref":"Skema.Result.html#new/1","title":"Skema.Result.new/1","type":"function"},{"doc":"Put valid data to result.","ref":"Skema.Result.html#put_data/3","title":"Skema.Result.put_data/3","type":"function"},{"doc":"Put error to result.","ref":"Skema.Result.html#put_error/3","title":"Skema.Result.put_error/3","type":"function"},{"doc":"Borrow from https://github.com/ejpcmac/typed_struct/blob/main/lib/typed_struct.ex","ref":"Skema.Schema.html","title":"Skema.Schema","type":"module"},{"doc":"Defines a typed struct.\n\nInside a `def_schema` block, each field is defined through the `field/2`\nmacro.","ref":"Skema.Schema.html#def_schema/2","title":"Skema.Schema.def_schema/2","type":"macro"},{"doc":"defmodule MyStruct do\n      use Skema.Schema\n\n      def_schema do\n        field :field_one, :string\n        field :field_two, :integer, required: true\n        field :field_three, :boolean, required: true\n        field :field_four, :atom, default: :hey\n        field :update_time, :naive_datetime, default: &NaiveDateTime.utc_now/0\n      end\n    end\n\nYou can create the struct in a submodule instead:\n\n    defmodule MyModule do\n      use Skema.Schema\n\n      def_schema Comment do\n        field :user_id, :integer, required: true\n        field :content, :string, required: true\n      end\n\n      def_schema Post do\n        field :field_one, :string\n        field :field_two, :integer, required: true\n        field :field_three, :boolean, required: true\n        field :field_four, :string, default: \"hello\"\n        field :update_time, :naive_datetime, default: &NaiveDateTime.utc_now/0\n        field :comment, Comment, required: true\n      end\n    end\n\n    MyModule.Post.cast(%{field_two: 1, field_three: true, comment: %{user_id: 1, content: \"hello\"}})","ref":"Skema.Schema.html#def_schema/2-examples","title":"Examples - Skema.Schema.def_schema/2","type":"macro"},{"doc":"Defines a field in a typed struct.","ref":"Skema.Schema.html#field/3","title":"Skema.Schema.field/3","type":"macro"},{"doc":"# A field named :example of type String.t()\n    field :example, String.t()","ref":"Skema.Schema.html#field/3-example","title":"Example - Skema.Schema.field/3","type":"macro"},{"doc":"* `default` - sets the default value for the field\n  * `enforce` - if set to true, enforces the field and makes its type\n    non-nullable","ref":"Skema.Schema.html#field/3-options","title":"Options - Skema.Schema.field/3","type":"macro"},{"doc":"This is only a specification to define a schema to use with `Skema.Params.cast` and `Skema.Contract.validate`\n\nSchema is just a map or keyword list that follow some simple conventions. Map's key is the field name and the value is a keyword list of field specifications.\n\n**Example**\n\n```elixir\n%{\n  name: [type: :string, format: ~r/{4}/],\n  age: [type: :integer, number: [min: 15, max: 50]].\n  skill: [type: {:array, :string}, length: [min: 1, max: 10]]\n}\n```\n\nIf you only specify type of data, you can write it shorthand style like this\n\n```elixir\n%{\n  name: :string,\n  age: :integer,\n  skill: {:array, :string}\n}\n```","ref":"Skema.SchemaHelper.html","title":"Skema.SchemaHelper","type":"module"},{"doc":"**Built-in types**\n\nA type could be any of built-in supported types:\n\n- `boolean`\n- `string` | `binary`\n- `integer`\n- `float`\n- `number` (integer or float)\n- `date`\n- `time`\n- `datetime` | `utc_datetime`: date time with time zone\n- `naive_datetime`: date time without time zone\n- `map`\n- `keyword`\n- `{array, type}` array of built-in type, all item must be the same type\n\n\n**Other types**\nCustom type may be supported depends on module.\n\n**Nested types**\nNested types could be a another **schema** or list of **schema**\n\n\n```elixir\n%{\n  user: [type: %{\n      name: [type: :string]\n    }]\n}\n```\n\nOr list of schema\n\n```elixir\n%{\n  users: [type: {:array, %{\n      name: [type: :string]\n    }} ]\n}\n```\n\n**Alias**\nAlias allow set a new key for value when using with `Taram.cast`\n\n```elixir\nschema = %{\n  email: [type: :string, as: :user_email]\n}\n\nSkema.cast(%{email: \"xx@yy.com\"}, schema)\n#> {:ok, %{user_email: \"xx@yy.com\"}}\n```","ref":"Skema.SchemaHelper.html#module-i-field-type","title":"I. Field type - Skema.SchemaHelper","type":"module"},{"doc":"These specifications is used for casting data with `Skema.Params.cast`\n\n#","ref":"Skema.SchemaHelper.html#module-ii-field-casting-and-default-value","title":"II. Field casting and default value - Skema.SchemaHelper","type":"module"},{"doc":"Is used when the given field is missing or nil.\n\n- Default could be a value\n\n  ```elixir\n  %{\n    status: [type: :string, default: \"active\"]\n  }\n  ```\n\n- Or a `function/0`, this function will be invoke each time data is `casted`\n\n  ```elixir\n  %{\n    published_at: [type: :datetime, default: &DateTime.utc_now/0]\n  }\n  ```\n\n#","ref":"Skema.SchemaHelper.html#module-1-default-value","title":"1. Default value - Skema.SchemaHelper","type":"module"},{"doc":"You can provide a function to cast field value instead of using default casting function by using\n`cast_func:  `\n\n```elixir\n%{\n    published_at: [type: :datetime, cast_func: &DateTime.from_iso8601/1]\n}\n```","ref":"Skema.SchemaHelper.html#module-2-custom-cast-function","title":"2. Custom cast function - Skema.SchemaHelper","type":"module"},{"doc":"**These validation are supported by `Skema.Validator`**\n\n#","ref":"Skema.SchemaHelper.html#module-iii-field-validation","title":"III. Field validation - Skema.SchemaHelper","type":"module"},{"doc":"Type specification above could be used for validating or casting data.\n\n#","ref":"Skema.SchemaHelper.html#module-1-type-validation","title":"1. Type validation - Skema.SchemaHelper","type":"module"},{"doc":"Support validating number value. These are list of supported validations:\n\n  - `equal_to`\n  - `greater_than_or_equal_to` | `min`\n  - `greater_than`\n  - `less_than`\n  - `less_than_or_equal_to` | `max`\n\n Define validation: `number: [ :  , ...]`\n\n**Example**\n\n```elixir\n%{\n  age: [type: :integer, number: [min: 1, max: 100]]\n}\n```\n\n#","ref":"Skema.SchemaHelper.html#module-2-numeric-validation","title":"2. Numeric validation - Skema.SchemaHelper","type":"module"},{"doc":"Validate length of supported types include `string`, `array`, `map`, `tuple`, `keyword`.\nLength condions are the same with **Numeric validation**\n\nDefine validation: `length: [ :  , ...]`\n\n**Example**\n\n```elixir\n%{\n  skills: [type: {:array, :string}, length: [min: 0, max: 5]]\n}\n```\n\n\n#","ref":"Skema.SchemaHelper.html#module-3-length-validation","title":"3. Length validation - Skema.SchemaHelper","type":"module"},{"doc":"Check if a string match a given pattern.\n\nDefine validation: `format:  `\n\n**Example**\n\n```elixir\n%{\n  year: [type: :string, format: ~r/year: {4}/]\n}\n```\n\n#","ref":"Skema.SchemaHelper.html#module-4-format-validation","title":"4. Format validation - Skema.SchemaHelper","type":"module"},{"doc":"Check if value is included or not included in given enumerable (`array`, `map`, or `keyword`)\n\nDefine validation: `in:  ` or `not_in:  `\n\n**Example**\n\n```elixir\n%{\n  status: [type: :string, in: [\"active\", \"inactive\"]],\n  selected_option: [type: :integer, not_in: [2,3]]\n}\n```\n\n#","ref":"Skema.SchemaHelper.html#module-5-inclusion-and-exclusion-validation","title":"5. Inclusion and exclusion validation - Skema.SchemaHelper","type":"module"},{"doc":"You can provide a function to validate the value.\n\nDefine validation: `func:  `\n\nFunction must be follow this signature\n\n```elixir\n@spec func(value::any()) :: :ok | {:error, message::String.t()}\n```","ref":"Skema.SchemaHelper.html#module-6-custom-validation-function","title":"6. Custom validation function - Skema.SchemaHelper","type":"module"},{"doc":"Expand short-hand type syntax to full syntax\n\n    field: :string -> field: [type: :string]\n    field: {:array, :string} -> field: [type: {:array, :string}]\n    field: %{#embedded} -> field: [type: %{#embedded}]","ref":"Skema.SchemaHelper.html#expand/1","title":"Skema.SchemaHelper.expand/1","type":"function"},{"doc":"Cast data to target type.\nMuch code of this module is borrowed from `Ecto.Type`","ref":"Skema.Type.html","title":"Skema.Type","type":"module"},{"doc":"","ref":"Skema.Type.html#cast/2","title":"Skema.Type.cast/2","type":"function"},{"doc":"","ref":"Skema.Type.html#cast!/2","title":"Skema.Type.cast!/2","type":"function"},{"doc":"# Skema\n\nPhoenix request params validation library.\n\n[![Build Status](https://github.com/bluzky/skema/workflows/Elixir%20CI/badge.svg)](https://github.com/bluzky/skema/actions) [![Coverage Status](https://coveralls.io/repos/github/bluzky/skema/badge.svg?branch=master)](https://coveralls.io/github/bluzky/skema?branch=master) [![Hex Version](https://img.shields.io/hexpm/v/skema.svg)](https://hex.pm/packages/skema) [![docs](https://img.shields.io/badge/docs-hexpm-blue.svg)](https://hexdocs.pm/skema/)\n\n\n\n- [Skema](#skema)\n    - [Why Skema](#why-skema)\n    - [Installation](#installation)\n    - [Usage](#usage)\n    - [Define schema](#define-schema)\n        - [Default value](#default-value)\n        - [Custom cast function](#custom-cast-function)\n            - [1. Custom cast function accept value only](#1-custom-cast-fuction-accept-value-only)\n            - [2. Custom cast function accept value and current object](#2-custom-cast-function-accept-value-and-current-object)\n            - [3.Custom cast function accept tuple {M, f}](#3custom-cast-function-accept-tuple-m-f)\n        - [Nested schema](#nested-schema)\n    - [Transform data](#transform-data)\n        - [Field name alias](#field-name-alias)\n        - [Convert data](#convert-data)\n    - [Validation](#validation)\n    - [Contributors](#contributors)","ref":"readme.html","title":"Skema","type":"extras"},{"doc":"- Reduce code boilerplate \n    - Shorter schema definition\n    - Default function which generate value each casting time\n    - Custom validation functions\n    - Custom parse functions","ref":"readme.html#why-skema","title":"Why Skema - Skema","type":"extras"},{"doc":"[Available in Hex](https://hex.pm/skema), the package can be installed\nby adding `skema` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:skema, \"~> 1.0.0\"}\n  ]\nend\n```","ref":"readme.html#installation","title":"Installation - Skema","type":"extras"},{"doc":"**Process order**\n> Cast data -> validate casted data -> transform data\n\n```elixir\n@index_params_schema  %{\n    keyword: :string,\n    status: [type: :string, required: true],\n    group_id: [type: :integer, number: [greater_than: 0]],\n    name: [type: :string, from: :another_field]\n  }\n\ndef index(conn, params) do\n    with {:ok, better_params} <- Skema.cast(params, @index_params_schema) do\n        # do anything with your params\n    else\n        {:error, errors} -> # return params error\n    end\nend\n```","ref":"readme.html#usage","title":"Usage - Skema","type":"extras"},{"doc":"Schema is just a map and it can be nested. Each field is defined as\n\n` : [ , ...]`\n\nOr short form\n\n` :  `\n\nField specs is a keyword list thay may include:\n\n- `type` is required, `Skema` support same data type as `Ecto`. I borrowed code from Ecto\n- `default`: default value or default function\n- `cast_func`: custom cast function\n- `number, format, length, in, not_in, func, required, each` are available validations\n- `from`: use value from another field\n- `as`: alias key you will receive from `Skema.cast` if casting is succeeded\n\n\n#","ref":"readme.html#define-schema","title":"Define schema - Skema","type":"extras"},{"doc":"You can define a default value for a field if it's missing from the params.\n\n```elixir\nschema = %{\n    status: [type: :string, default: \"pending\"]\n}\n```\n\nOr you can define a default value as a function. This function is evaluated when `Skema.cast` gets invoked.\n\n```elixir\nschema = %{\n    date: [type: :utc_datetime, default: &Timex.now/0]\n}\n```\n\n#","ref":"readme.html#default-value","title":"Default value - Skema","type":"extras"},{"doc":"You can define your own casting function, `skema` provide `cast_func` option.\nYour `cast_func` must follows this spec \n\n##","ref":"readme.html#custom-cast-function","title":"Custom cast function - Skema","type":"extras"},{"doc":"```elixir\nfn(any) :: {:ok, any} | {:error, binary} | :error\n```\n\n```elixir\ndef my_array_parser(value) do\n    if is_binary(value) do\n        ids = \n            String.split(value, \",\")\n            |> Enum.map(&String.to_integer(&1))\n        \n        {:ok, ids}\n    else\n        {:error, \"Invalid string\"\n    end\nend\n\nschema = %{\n    user_id: [type: {:array, :integer}, cast_func: &my_array_parser/1]\n}\n\nSkema.cast(%{user_id: \"1,2,3\"}, schema)\n```\nThis is a demo parser function.\n\n##","ref":"readme.html#1-custom-cast-fuction-accept-value-only","title":"1. Custom cast fuction accept value only - Skema","type":"extras"},{"doc":"```elixir\ndata = %{\n   name: \"tada\",\n   bold: true\n}\n\nschema = %{\n    name: [type: :string, cast_func: fn value, data -> \n        {:ok, (if data.bold, do: String.upcase(value), else: value)}\n    end]\n}\n\nSkema.cast(data, schema)\n\n# > %{name: \"TADA\"}\n```\n\n##","ref":"readme.html#2-custom-cast-function-accept-value-and-current-object","title":"2. Custom cast function accept value and current object - Skema","type":"extras"},{"doc":"Your cast function must accept 2 arguments\n\n```elixir\ndefmodule MyModule do\n    def upcase(value, data) do\n        {:ok, (if data.bold, do: String.upcase(value), else: value)}\n    end\nend\n```\n\n```elixir\ndata = %{\n   name: \"tada\",\n   bold: true\n}\n\nschema = %{\n    name: [type: :string, cast_func: {MyModule, :upcase}]\n}\n\nSkema.cast(data, schema)\n\n# > %{name: \"TADA\"}\n```\n\n\n#","ref":"readme.html#3-custom-cast-function-accept-tuple-m-f","title":"3.Custom cast function accept tuple {M, f} - Skema","type":"extras"},{"doc":"With `Skema` you can parse and validate nested map and list easily\n\n```elixir\n@my_schema %{\n    status: :string,\n    pagination: %{\n        page: [type: :integer, number: [min: 1]],\n        size: [type: :integer, number: [min: 10, max: 100\"]]\n    }\n}\n```\n\nOr nested list schema\n\n```elixir\n@user_schema %{\n    name: :string,\n    email: [type: :string, required: true]\n    addresses: [type: {:array, %{\n        street: :string,\n        district: :string,\n        city: :string\n    }}]\n}\n```","ref":"readme.html#nested-schema","title":"Nested schema - Skema","type":"extras"},{"doc":"`Skema` uses `Valdi` validation library. You can read more about [Valdi here](https://github.com/bluzky/valdi)\nBasically it supports following validation\n\n- validate inclusion/exclusion\n- validate length for string and enumerable types\n- validate number\n- validate string format/pattern\n- validate custom function\n- validate required(not nil) or not\n- validate each array item\n\n\n\n  ```elixir\n  product_schema = %{\n    sku: [type: :string, required: true, length: [min: 6, max: 20]]\n    name: [type: :string, required: true],\n    quantity: [type: :integer, number: [min: 0]],\n    type: [type: :string, in: ~w(physical digital)],\n    expiration_date: [type: :naive_datetime, func: &my_validation_func/1],\n    # dynamic required\n    width: [type: :integer, required: fn value, data -> data.type == \"physical\" end],\n    # validate each array item\n    tags: [type: {:array, :string}, each: [length: [max: 50]]]\n  }\n  ```\n\n#","ref":"readme.html#validation","title":"Validation - Skema","type":"extras"},{"doc":"- Can accept function or `{module, function}` tuple\n- Only support 2 arity function\n\n\n```elixir\ndef require_email?(value, data), do: is_nil(email.phone)\n\n....\n\n%{\n    phone: :string\n    name: [type: :string, required: fn value, data -> true end],\n    email: [type: :string, required: {__MODULE__, :require_email?}]\n}\n```\n\n#","ref":"readme.html#dynamic-required","title":"Dynamic required - Skema","type":"extras"},{"doc":"Support validate array item with `:each` option, `each` accept a list of validators\n\n```elixir\n%{\n    values: [type: {:array, :number}, each: [number: [min: 20, max: 50]]]\n  }\n```","ref":"readme.html#validate-array-item","title":"Validate array item - Skema","type":"extras"},{"doc":"#","ref":"readme.html#transform-data","title":"Transform data - Skema","type":"extras"},{"doc":"You can set alias name for schema fields\n```elixir\ndata = %{\n   name: \"tada\"\n}\n\nschema = %{\n    name: [type: :string, as: :full_name]\n}\n\nSkema.cast(data, schema)\n\n# > %{full_name: \"tada\"}\n```\n\n#","ref":"readme.html#field-name-alias","title":"Field name alias - Skema","type":"extras"},{"doc":"You can specify a function similar to `cast_func` to manipulate data after casted.\nHowever data object passed to transform function is original data before casting.\n\n```elixir\ndata = %{status: 10}\n\nschema = %{\n    name: [type: :string, into: fn value -> {:ok, \"name: #{value}}\" end]\n}\n\nSkema.cast(data, schema)\n# > %{name: \"name: tada\"}\n\n```\n\n- Transform function can return tuple `{:ok, value}`, `{:error, message}` or value directly.\n\n```elixir\nschema = %{\n    value: [type: :integer, into: &to_string/1]\n}\n```","ref":"readme.html#convert-data","title":"Convert data - Skema","type":"extras"},{"doc":"If you find a bug or want to improve something, please send a pull request. Thank you!","ref":"readme.html#contributors","title":"Contributors - Skema","type":"extras"}]}